﻿local L = LibStub("AceLocale-3.0"):GetLocale("GridStatusPhysicalMitigation")

local GridRoster = Grid:GetModule("GridRoster")

local GridStatusPhysicalMitigation = Grid:GetModule("GridStatus"):NewModule("GridStatusPhysicalMitigation","AceTimer-3.0")
GridStatusPhysicalMitigation.menuName = L["Physical Mitigation"];
GridStatusPhysicalMitigation.options = false

local playerClass, englishClass = UnitClass("player")
local playerRace, englishRace = UnitRace("player")

local spellNameCache = {};
spellNameCache.AncestralFortitude = GetSpellInfo(16236);
spellNameCache.Inspiration = GetSpellInfo(15362);

local mainTankHash = {};

local refreshTimer;

--{{{ AceDB defaults
--
GridStatusPhysicalMitigation.defaultDB = {
	alert_physicalmitigation = {
		text = L["Physical Mitigation"],
		desc = L["Ancestral Fortitude / Inspiration"],
		enable = true,
		maintank = false,
		priority = 99,
		range = false,
		threshold2 = 6,
		threshold3 = 3,
		color = { r = 0, g = 1, b = 0, a = 1 },
		color2 = { r = 1, g = 1, b = 0, a = 1 },
		color3 = { r = 1, g = 0, b = 0, a = 1 },
	},
}

local physicalMitigationOptions = {
	["threshold2"] = {
		type = "range",
		name = L["Threshold to activate color 2"],
		desc = L["Threshold to activate color 2"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
				return GridStatusPhysicalMitigation.db.profile.alert_physicalmitigation.threshold2
			end,
		set = function (_, v)
			GridStatusPhysicalMitigation.db.profile.alert_physicalmitigation.threshold2 = v
		end,
		order = 90,
	},
	["color2"] = {
		type = "color",
		name = L["Color 2"],
		desc = L["Color 2"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusPhysicalMitigation.db.profile.alert_physicalmitigation.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (_, r, g, b, a)
			local color = GridStatusPhysicalMitigation.db.profile.alert_physicalmitigation.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
		order = 90,
	},
	["threshold3"] = {
		type = "range",
		name = L["Threshold to activate color 3"],
		desc = L["Threshold to activate color 3"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			return GridStatusPhysicalMitigation.db.profile.alert_physicalmitigation.threshold3
		end,
		set = function (_, v)
			GridStatusPhysicalMitigation.db.profile.alert_physicalmitigation.threshold3 = v
		end,
		order = 90,
	},
	["color3"] = {
		type = "color",
		name = L["Color 3"],
		desc = L["Color 3"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusPhysicalMitigation.db.profile.alert_physicalmitigation.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (_, r, g, b, a)
			local color = GridStatusPhysicalMitigation.db.profile.alert_physicalmitigation.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
		order = 90,
	},
	["maintank"] = {
		type = "toggle",
		name = L["Show for Main Tanks only"],
		desc = name,
		order = 90,
		get = function ()
			      return GridStatusPhysicalMitigation.db.profile.alert_physicalmitigation.maintank
		      end,
		set = function (_, v)
			      GridStatusPhysicalMitigation.db.profile.alert_physicalmitigation.maintank = v
		      end,
	},
}

function GridStatusPhysicalMitigation:OnInitialize()
	self.super.OnInitialize(self)
	if (englishClass == "SHAMAN" or englishClass == "PRIEST") then
		self:RegisterStatus("alert_physicalmitigation", L["Physical Mitigation"], physicalMitigationOptions,true)
	end
end

function GridStatusPhysicalMitigation:OnEnable()
	if not GridRoster then
		RL = AceLibrary("Roster-2.1")
	end
	
	if self.TimerSet == true then
		self:CancelTimer(refreshTimer, true)
	end
	refreshTimer = self:ScheduleRepeatingTimer("UpdateAll", 0)
	self.TimerSet = true
	
--	self:ScheduleRepeatingEvent("GPM_Refresh", self.UpdateAll, GridStatusPhysicalMitigation.db.profile.frequency, self)

	--if true then
		self:RegisterEvent("oRA_MainTankUpdate", "UpdateMainTankTable")
		self:RegisterEvent("RAID_ROSTER_UPDATE", "UpdateMainTankTable")
		if CT_RAOptions_UpdateMTs then
			hooksecurefunc("CT_RAOptions_UpdateMTs", GridStatusMTs.UpdateMainTankTable)
		end
		self:UpdateMainTankTable()
		
		self:RegisterEvent("PLAYER_ROLES_ASSIGNED", "RoleCheck")
		self:RegisterEvent("PARTY_MEMBERS_CHANGED", "RoleCheck")
		self:RoleCheck()
	--end
end

function GridStatusPhysicalMitigation:UpdateFrequency()
	if self.TimerSet == true then
		self:CancelTimer(refreshTimer, true)
	end
	refreshTimer = self:ScheduleRepeatingTimer("UpdateAll", 0)
	self.TimerSet = true
	
--	self:CancelScheduledEvent("GPM_Refresh");
--	self:ScheduleRepeatingEvent("GPM_Refresh", self.UpdateAll, GridStatusPhysicalMitigation.db.profile.frequency, self)
end

function GridStatusPhysicalMitigation:Reset()
	self.super.Reset(self)
	
	-- unregister status event
	-- register status event
	-- update all
end

local function updateUnit(self, gridid, unitid)
	local pmtime
	local now = GetTime()
	-- Find number of hots per unit
	for i=1,999 do
		local bname,brank,btexture,bcount,btype,bdur,bexptime,bismine = UnitBuff(unitid, i)
		if not bname then break end
		
		-- if tanks are on, and this is not a tank, do nothing
		if (self.db.profile.alert_physicalmitigation.maintank and 
			not mainTankHash[gridid] )
		then
			if self.core:GetCachedStatus(gridid, "alert_physicalmitigation") then
				self.core:SendStatusLost(gridid, "alert_physicalmitigation")
			end
		return end
		
		local btime = bexptime - now
		--PhysicalMitigation
		if (bname == spellNameCache["AncestralFortitude"] or bname == spellNameCache["Inspiration"]) then
			if (btime) then
				pmtime = btime
			end
		end

		if pmtime and self.db.profile.alert_physicalmitigation.enable then
			local settings = self.db.profile.alert_physicalmitigation
			local hotcolor = settings.color
			if pmtime <= settings.threshold2 then hotcolor = settings.color2 end
			if pmtime <= settings.threshold3 then hotcolor = settings.color3 end

			self.core:SendStatusGained(gridid, "alert_physicalmitigation",
				settings.priority,
				(settings.range and 40),
				hotcolor,
				tostring(math.floor(pmtime) )
			)
		else
			if self.core:GetCachedStatus(gridid, "alert_physicalmitigation") then
				self.core:SendStatusLost(gridid, "alert_physicalmitigation")
			end
		end
	end
end

function GridStatusPhysicalMitigation:UpdateAll()
	if GridRoster then
		for guid, unitid in GridRoster:IterateRoster() do
			updateUnit(self, guid, unitid)
		end
	else
		for u in RL:IterateRoster(true) do
			local name = UnitName(u.unitid)
			if name then
				updateUnit(self, name, u.unitid)
			end
		end
	end
end

function GridStatusPhysicalMitigation:UpdateMainTankTable()
	local settings = self.db.profile.alert_physicalmitigation
	if not settings.maintank then return end -- needed for CT_RA hooksecurefunc
	
	-- clear the MT list
	mainTankHash = {}
	local mainTankTable
	local isBlizzMTs -- nil/false = ADDON | true = BLIZZARD
	if oRA then
		mainTankTable = oRA.maintanktable
	end
	if CT_RA_MainTanks then
		mainTankTable = CT_RA_MainTanks
	end
	if not mainTankTable then
		mainTankTable = {}
		local x = 1
		local raid = GetNumRaidMembers()
		if raid > 0 then
			for i = 1, raid do
				local name, _, _, _, _, _, _, _, _, role = GetRaidRosterInfo(i)
				if name and role and role == "MAINTANK" then
					mainTankTable[x] = name
					x = x + 1
					if x > 10 then
						break
					end
				end
			end
		end
		if mainTankTable[1] then
			isBlizzMTs = true -- nil/false = ADDON | true = BLIZZARD
		else
			mainTankTable = nil
		end
	end	
	
	if not mainTankTable then return end
	
	for i = 1, 10 do
		if mainTankTable[i] then

			for guid, unitid in GridRoster:IterateRoster() do
				local name = GridRoster:GetNameByGUID(guid)
				if mainTankTable[i] == name then
					mainTankHash[guid] = name;
				end
			end

		end
	end
end

-- for each party member, if they are marked as a tank role, put them in the main tank list
function GridStatusPhysicalMitigation:RoleCheck()
    local settings = self.db.profile.alert_physicalmitigation
	if not settings.maintank then return end 

	for guid in GridRoster:IterateRoster() do
		local isTank, isHeal, isDPS = UnitGroupRolesAssigned(GridRoster:GetUnitidByGUID(guid))
		if isTank then
			local name = GridRoster:GetNameByGUID(guid)
			mainTankHash[guid] = name;
		end
	end
end